#!/usr/bin/perl -w
# dbstat.pl
#
# This module retrieves transaction statistics of one or more database servers.
# The results are sent to the parent rspd.
# 
# Currently, this module only supports the MySQL database (type = 'mysql').
# 
# Copyright 2004 Draconis Software, LLC <www.dracoware.com>

BEGIN {
	my $err = '';
	my @mods = ('DBI', 'rspmod');
	foreach my $mod (@mods) {
		eval "use $mod";
		if($@) {
			$err .= "Couldn't find required module $mod.pm.\n";
		}
	}
	
	if($err ne '') {
		$err .= "\nIncluded directories: @INC\n\n";
		die $err;
	}
}

use DBI;
use rspmod;

# --------------------------------------------------------------------------- #
# Configuration Parameters                                                    #
# --------------------------------------------------------------------------- #
my %servers;
my $shorten_suffix = 0;                 # 1 = 'q/s', 0 = 'queries per second' #
# --------------------------------------------------------------------------- #

my $gError;

# --------------------------------------------------------------------------- #
# main - setup and run the script                                             #
# --------------------------------------------------------------------------- #
sub main()
{
	my $mod = new rspmod::RSP_moduleBackend("dbstat");
	my %last_queries;

	# get the configuration options

	# should we shorten the suffix?
	if(defined($mod->getConfigValue("shorten_suffix")) &&
		$mod->getConfigValue("shorten_suffix") =~ /1|on|yes/) {
		$shorten_suffix = 1;
	}

	# Site settings are defined by first listing a site in
	# the "site" variable.  Then the host, type, user and pass
	# are set by appending them to the site name.  For example:
	#
	# site = localhost1
	# localhost1.host = localhost
	# localhost1.type = mysql
	# localhost1.user = root
	# localhost1.pass = password
	my $sites = $mod->getConfig("site");
	if(!defined($sites)) {
		$mod->log(1, "No sites defined.  Quitting.");
		$mod->cleanup();
		exit();
	}

	my $currsite;
	while(defined(($currsite = $sites->nextConfigData()))) {
		foreach $setting ("host", "type", "user", "pass") {
			if(!defined($mod->getConfigValue("$currsite.$setting"))) {
				$mod->log(1, "$currsite.$setting not defined.  Quitting.");
				$mod->cleanup();
				exit();
			}

			$servers{$currsite}{$setting} = $mod->getConfigValue("$currsite.$setting");
		}
	}

	while(1)
	{
		my $returnVal;
		my $out;
		
		# update the queries
		foreach my $key (keys %servers)
		{
			my $qps = 0.0;
			my $queries = get_queries($key);

			if($queries == -1)
			{
				if(not $out) {
					$out = "$key: $gError";
				}
				else {
					$out .= ", $key: $gError";
				}

				$last_queries{$key} = undef;

				# check if any thresholds were set
#				while(defined(my $threshold = $mod->nextThreshold()))
#				{
#					if($threshold =~ /$key/) {
#						$mod->setThreshold($threshold, 0);
#					}
#				}
			}
			else
			{
				my $firstRun;
				$firstRun = 1 if(not defined $last_queries{$key});
				
				# calculate the queries per second
				$qps = ($queries - $last_queries{$key}) / $mod->getPolltime()
					if $last_queries{$key};
				
				$last_queries{$key} = $queries;
				
				my $suffix = "queries per second";
				$suffix = "q/s" if $shorten_suffix;
				
				if(not $out) {
					$out = "$key: $qps $suffix";
				}
				else {
					$out .= ", $key: $qps $suffix";
				}
				
				if(not $firstRun)
				{
					# create the graphdata
					my $graphData = new rspmod::RSP_graphReturnData(0, "qps_$key", "$qps");
					$mod->giveGraphData($graphData);
	
					# check if we crossed a threshold
					while(defined(my $threshold = $mod->nextThreshold()))
					{
						if($threshold =~ /^\s*$key\s*([!<>=]=?)\s*(.+)\s*$/)
						{
							my $operator = $1;
							my $num = $2;
							
							# check the value
							if(($num =~ /^-?(?:\d+(?:\.\d*)?|\.\d+)$/) && ($num >= 0))
							{
								if($operator eq "<") {
									$mod->setThreshold($threshold, 0) if $qps < $num;
								} elsif($operator eq "<=") {
									$mod->setThreshold($threshold, 0) if $qps <= $num;
								} elsif($operator eq ">") {
									$mod->setThreshold($threshold, 0) if $qps > $num;
								} elsif($operator eq ">=") {
									$mod->setThreshold($threshold, 0) if $qps >= $num;
								} elsif($operator eq "!=") {
									$mod->setThreshold($threshold, 0) if $qps != $num;
								} elsif(($operator eq "==") || ($operator eq "=")) {
									$mod->setThreshold($threshold, 0) if $qps == $num;
								}
							} # if(($num =~ /^-?(?:\d+(?:\.\d*)?|\.\d+)$/) && ($num >= 0))
						} # if($threshold =~ /^\s*$key\s*([!<>=]=?)\s*(.+)\s*$/)
					} # while(defined(my $threshold = $mod->nextThreshold()))
				} # if(not $firstRun)
			} # if($queries != -1)
		} # foreach my $key (keys %servers)
		
		$mod->giveData($out);
		
		eval { $mod->postMessage(); };
		exit if $@;
	
		$out = undef;
		
		# wait until there is an event, or the specified iteration period
		eval {
			$returnVal = $mod->waitNextEvent($mod->getPolltime());
		};
		if ($@) {
			if ($returnVal == -1) {
				$mod->log(1, "Message from rspd was not recieved correctly.  Quitting.");
			}

			$mod->cleanup();
			exit();
		}
	}
}


# --------------------------------------------------------------------------- #
# get_queries - returns the number of queries currently asked of the db       #
# --------------------------------------------------------------------------- #
sub get_queries($)
{
	my $key = shift @_;

	my $type = $servers{$key}{"type"};
	my $host = $servers{$key}{"host"};
	my $user = $servers{$key}{"user"};
	my $pass = $servers{$key}{"pass"};
	
	# connect to the database
	my $dbh = DBI->connect("DBI:$type:host=$host", "$user", "$pass", 
		{'RaiseError' => 0, 'PrintError' => 0});
	if(not $dbh) {
		$gError = $DBI::errstr;
		return -1;
	}
	
	# get the queries per second data
	my $sth = $dbh->prepare('SHOW STATUS LIKE "Questions"');
	$sth->execute();
	
	my $qps;
	while(my $ref = $sth->fetchrow_hashref) {
		$qps = $ref->{Value};
	}
	
	# cleanup
	$sth->finish();
	$dbh->disconnect();
	
	return $qps;
}

main();

